{-# LANGUAGE DerivingVia #-}
{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE DataKinds #-}
module Language.Java.Classfile.ConstantPool.Entry (Entry(..)) where
import GHC.Generics (Generic, Generically(..))
import Language.Java.Classfile.Extractable (Extractable)
import Language.Java.Classfile.Extractable.WithTag (Word8Tag)
import Language.Java.Classfile.ConstantPool.References (Utf8Reference, ClassReference, NameAndTypeReference, MethodHandleReferenceKind, OpaqueReference, BootstrapMethodIndex)
import Data.Int (Int32, Int64)

data Entry 
  = Integer             (Word8Tag 3  Int32)
  | Float               (Word8Tag 4  Float)
  | Long                (Word8Tag 5  Int64)
  | Double              (Word8Tag 6  Double)
  | Class               (Word8Tag 7  Utf8Reference)                                 -- name
  | String              (Word8Tag 8  Utf8Reference)                                 -- value
  | FieldRef            (Word8Tag 9  (ClassReference, NameAndTypeReference))        -- containing class, descriptor
  | MethodRef           (Word8Tag 10 (ClassReference, NameAndTypeReference))        -- containing class, descriptor
  | InterfaceMethodRef  (Word8Tag 11 (ClassReference, NameAndTypeReference))        -- containing class, descriptor
  | NameAndType         (Word8Tag 12 (Utf8Reference, Utf8Reference))                -- name, type
  | MethodHandle        (Word8Tag 15 MethodHandleInfo)
  | MethodType          (Word8Tag 16 Utf8Reference)                                 -- descriptor
  | Dynamic             (Word8Tag 17 (BootstrapMethodIndex, NameAndTypeReference))  -- index into the bootstrapmethod attribute array, NameAndType must refer to a field
  | InvokeDynamic       (Word8Tag 18 (BootstrapMethodIndex, NameAndTypeReference))  -- index into the bootstrapmethod attribute array, NameAndType must refer to a method
  | Module              (Word8Tag 19 Utf8Reference)
  | Package             (Word8Tag 20 Utf8Reference)
  deriving stock (Show, Generic)
  deriving Extractable via Generically Entry

data MethodHandleInfo = MethodHandleInfo MethodHandleReferenceKind OpaqueReference
  deriving stock (Show, Generic)
  deriving Extractable via Generically MethodHandleInfo
